% FundamentalEightPoint  The 8-point algorithm for the estimation of the fundamental matrix F
%
% The eight-point algorithm for the fundamental matrix with a posteriori
% enforcement of the singularity constraint (det(F)=0).
% Does not include data normalization.
%
% Reference: "Multiple View Geometry" (Hartley & Zisserman 2000), Sect. 10.1 page 262.
%
% Input: point correspondences
%  - x1(3,N): homogeneous coordinates of 2-D points in image 1
%  - x2(3,N): homogeneous coordinates of 2-D points in image 2
%
% Output:
%  - F(3,3) : fundamental matrix
%  - cost   : fitting cost


function [F,cost] = FundamentalEightPoint(x1,x2)

[dim,NumPoints] = size(x1);
[dim2,NumPoints2] = size(x2);

% Sanity checks
assert(dim==dim2 && NumPoints==NumPoints2,'Size mismatch of input points');
assert(dim==3,'Input arguments are not 2D points');
assert(NumPoints>=8,'Insufficient number of points to compute fundamental matrix (need >=8)');

% Compute the measurement matrix A of the linear homogeneous system whose
% solution is the vector representing the fundamental matrix.
A = ; % Initialize matrix of size (NumPoints x 9) with zeros.
for i=1:NumPoints
    % Compute the kronercker product (kron function) between the i-th
    % correspondences x2 and x1, in this order, and make it a row vector
    % since we are filling in the rows of matrix A.
    A(i,:) = ; 
end

% "Solve" the linear homogeneous system of equations A*f = 0.
% The correspondences x1,x2 are exact <=> rank(A)=8 -> there exist an exact solution
% If measurements are noisy, then rank(A)=9 => there is no exact solution, seek a least-squares solution.
[~,D,V] = svd(A,0);
S = diag(D);
F = ; % convert the last column of V to the correct 3 x 3 matrix

if (NumPoints==8)
    cost=0;
else
    cost = S(9);
end
disp(['Smallest singular value = ',num2str(cost)]);

% Enforce det(F)=0 by projecting F onto the set of 3x3 singular matrices
F = F2SingularF(F);
