% LINEARTRIANGULATION  The Multicamera Linear Triangulation Method for n points
%
% Input:
%  - x1(3,N): homogeneous coordinates of points in image 1
%  - x2(3,N): homogeneous coordinates of points in image 2
%  - P1(3,4): projection matrix corresponding to first image
%  - P2(3,4): projection matrix corresponding to second image
%
% Output:
%  - X(4,N): homogeneous coordinates of 3-D points
%  - fitting_costs(N,1): fitting cost of the 3-D point
%
% Reference: Section 11.2 page 297 "Multiple View Geometry" Hartley & Zisserman 2000.


function [X,fitting_costs] = LinearTriangulation(x1,x2,P1,P2)

% Sanity checks
[dim,NumPoints] = size(x1);
[dim2,npoints2] = size(x2);
assert(dim==dim2,'Size mismatch of input points');
assert(NumPoints==npoints2,'Size mismatch of input points');
assert(dim==3,'Arguments x1, x2 should be 3xN matrices (homogeneous coords)');

[rows,cols] = size(P1);
assert(rows==3 && cols==4,'Projection matrices should be of size 3x4');
[rows,cols] = size(P2);
assert(rows==3 && cols==4,'Projection matrices should be of size 3x4');

X = zeros(4,NumPoints);
fitting_costs = zeros(NumPoints,1);

% Linear algorithm
for j=1:NumPoints
    % Built matrix of linear homogeneous system of equations using
    % projection matrices P1, P2 and image points x1, x2.
    A1 = ; % 3x4 matrix. Use call to skew-symmetric matrix in function Cross2Matrix
    A2 = ;
    A = ; % 6x4 system of equations formed by stacking A1 and A2
    
    % "Solve" the linear homogeneous system of equations
    [~,d,v] = svd(A,0);
    X(:,j) = v(:,4);
    fitting_costs(j) = d(4,4);
end
